@@ -18,13 +18,14 @@ use crate::{
     crate_metadata::CrateMetadata,
     maybe_println, util, validate_wasm,
     workspace::{Manifest, ManifestPath, Profile, Workspace},
-    BuildArtifacts, BuildResult, OptimizationPasses, OptimizationResult, UnstableFlags,
-    UnstableOptions, Verbosity, VerbosityFlags,
+    BuildArtifacts, BuildMode, BuildResult, OptimizationPasses, OptimizationResult, OutputType,
+    UnstableFlags, UnstableOptions, Verbosity, VerbosityFlags,
 };
 use anyhow::{Context, Result};
 use colored::Colorize;
-use parity_wasm::elements::{External, MemoryType, Module, Section};
+use parity_wasm::elements::{External, Internal, MemoryType, Module, Section};
 use regex::Regex;
+use semver::Version;
 use std::{
     convert::TryFrom,
     ffi::OsStr,
@@ -38,6 +39,20 @@ use structopt::StructOpt;
 /// This is the maximum number of pages available for a contract to allocate.
 const MAX_MEMORY_PAGES: u32 = 16;
 
+/// Arguments to use when executing `build` or `check` commands.
+#[derive(Default)]
+pub(crate) struct ExecuteArgs {
+    /// The location of the Cargo manifest (`Cargo.toml`) file to use.
+    pub(crate) manifest_path: ManifestPath,
+    verbosity: Verbosity,
+    build_mode: BuildMode,
+    build_artifact: BuildArtifacts,
+    unstable_flags: UnstableFlags,
+    optimization_passes: OptimizationPasses,
+    keep_debug_symbols: bool,
+    output_type: OutputType,
+}
+
 /// Executes build of the smart-contract which produces a wasm binary that is ready for deploying.
 ///
 /// It does so by invoking `cargo build` and then post processing the final binary.
@@ -47,6 +62,14 @@ pub struct BuildCommand {
     /// Path to the Cargo.toml of the contract to build
     #[structopt(long, parse(from_os_str))]
     manifest_path: Option<PathBuf>,
+    /// By default the contract is compiled with debug functionality
+    /// included. This enables the contract to output debug messages,
+    /// but increases the contract size and the amount of gas used.
+    ///
+    /// A production contract should always be build in `release` mode!
+    /// Then no debug functionality is compiled into the contract.
+    #[structopt(long = "--release")]
+    build_release: bool,
     /// Which build artifacts to generate.
     ///
     /// - `all`: Generate the Wasm, the metadata and a bundled `<name>.contract` file.
@@ -81,13 +104,22 @@ pub struct BuildCommand {
     ///
     /// - `z`, execute default optimization passes, super-focusing on code size
     ///
-    /// - The default value is `3`
+    /// - The default value is `z`
     ///
     /// - It is possible to define the number of optimization passes in the
     ///   `[package.metadata.contract]` of your `Cargo.toml` as e.g. `optimization-passes = "3"`.
     ///   The CLI argument always takes precedence over the profile value.
-    #[structopt(long = "optimization-passes")]
+    #[structopt(long)]
     optimization_passes: Option<OptimizationPasses>,
+    /// Do not remove symbols (Wasm name section) when optimizing.
+    ///
+    /// This is useful if one wants to analyze or debug the optimized binary.
+    #[structopt(long)]
+    keep_debug_symbols: bool,
+
+    /// Export the build output in JSON format.
+    #[structopt(long, conflicts_with = "verbose")]
+    output_json: bool,
 }
 
 impl BuildCommand {
@@ -95,7 +127,7 @@ impl BuildCommand {
         let manifest_path = ManifestPath::try_from(self.manifest_path.as_ref())?;
         let unstable_flags: UnstableFlags =
             TryFrom::<&UnstableOptions>::try_from(&self.unstable_options)?;
-        let verbosity = TryFrom::<&VerbosityFlags>::try_from(&self.verbosity)?;
+        let mut verbosity = TryFrom::<&VerbosityFlags>::try_from(&self.verbosity)?;
 
         // The CLI flag `optimization-passes` overwrites optimization passes which are
         // potentially defined in the `Cargo.toml` profile.
@@ -112,13 +144,33 @@ impl BuildCommand {
             }
         };
 
-        execute(
-            &manifest_path,
+        let build_mode = match self.build_release {
+            true => BuildMode::Release,
+            false => BuildMode::Debug,
+        };
+
+        let output_type = match self.output_json {
+            true => OutputType::Json,
+            false => OutputType::HumanReadable,
+        };
+
+        // We want to ensure that the only thing in `STDOUT` is our JSON formatted string.
+        if matches!(output_type, OutputType::Json) {
+            verbosity = Verbosity::Quiet;
+        }
+
+        let args = ExecuteArgs {
+            manifest_path,
             verbosity,
-            self.build_artifact,
+            build_mode,
+            build_artifact: self.build_artifact,
             unstable_flags,
             optimization_passes,
-        )
+            keep_debug_symbols: self.keep_debug_symbols,
+            output_type,
+        };
+
+        execute(args)
     }
 }
 
@@ -140,13 +192,19 @@ impl CheckCommand {
         let unstable_flags: UnstableFlags =
             TryFrom::<&UnstableOptions>::try_from(&self.unstable_options)?;
         let verbosity: Verbosity = TryFrom::<&VerbosityFlags>::try_from(&self.verbosity)?;
-        execute(
-            &manifest_path,
+
+        let args = ExecuteArgs {
+            manifest_path,
             verbosity,
-            BuildArtifacts::CheckOnly,
+            build_mode: BuildMode::Debug,
+            build_artifact: BuildArtifacts::CheckOnly,
             unstable_flags,
-            OptimizationPasses::Zero,
-        )
+            optimization_passes: OptimizationPasses::Zero,
+            keep_debug_symbols: false,
+            output_type: OutputType::default(),
+        };
+
+        execute(args)
     }
 }
 
@@ -169,6 +227,7 @@ impl CheckCommand {
 fn exec_cargo_for_wasm_target(
     crate_metadata: &CrateMetadata,
     command: &str,
+    build_mode: BuildMode,
     verbosity: Verbosity,
     unstable_flags: &UnstableFlags,
 ) -> Result<()> {
@@ -178,19 +237,24 @@ fn exec_cargo_for_wasm_target(
     // Currently will override user defined RUSTFLAGS from .cargo/config. See https://github.com/paritytech/cargo-contract/issues/98.
     std::env::set_var(
         "RUSTFLAGS",
-        "-C link-arg=-z -C link-arg=stack-size=65536 -C link-arg=--import-memory",
+        "-C link-arg=-zstack-size=65536 -C link-arg=--import-memory",
     );
 
     let cargo_build = |manifest_path: &ManifestPath| {
         let target_dir = &crate_metadata.target_directory;
-        let args = [
+        let target_dir = format!("--target-dir={}", target_dir.to_string_lossy());
+        let mut args = vec![
             "--target=wasm32-unknown-unknown",
             "-Zbuild-std",
-            "-Zbuild-std-features=panic_immediate_abort",
             "--no-default-features",
             "--release",
-            &format!("--target-dir={}", target_dir.to_string_lossy()),
+            &target_dir,
         ];
+        if build_mode == BuildMode::Debug {
+            args.push("--features=ink_env/ink-debug");
+        } else {
+            args.push("-Zbuild-std-features=panic_immediate_abort");
+        }
         util::invoke_cargo(command, &args, manifest_path.directory(), verbosity)?;
 
         Ok(())
@@ -260,31 +324,43 @@ fn ensure_maximum_memory_pages(module: &mut Module, maximum_allowed_pages: u32)
 /// Strips all custom sections.
 ///
 /// Presently all custom sections are not required so they can be stripped safely.
+/// The name section is already stripped by `wasm-opt`.
 fn strip_custom_sections(module: &mut Module) {
-    module.sections_mut().retain(|section| {
-        !matches!(
-            section,
-            Section::Custom(_) | Section::Name(_) | Section::Reloc(_)
-        )
-    });
+    module.sections_mut().retain(|section| match section {
+        Section::Reloc(_) => false,
+        Section::Custom(custom) if custom.name() != "name" => false,
+        _ => true,
+    })
+}
+
+/// A contract should export nothing but the "call" and "deploy" functions.
+///
+/// Any elements not referenced by these exports become orphaned and are removed by `wasm-opt`.
+fn strip_exports(module: &mut Module) {
+    if let Some(section) = module.export_section_mut() {
+        section.entries_mut().retain(|entry| {
+            matches!(entry.internal(), Internal::Function(_))
+                && (entry.field() == "call" || entry.field() == "deploy")
+        })
+    }
+}
+
+/// Load and parse a wasm file from disk.
+fn load_module<P: AsRef<Path>>(path: P) -> Result<Module> {
+    let path = path.as_ref();
+    parity_wasm::deserialize_file(path).context(format!(
+        "Loading of wasm module at '{}' failed",
+        path.display(),
+    ))
 }
 
 /// Performs required post-processing steps on the wasm artifact.
 fn post_process_wasm(crate_metadata: &CrateMetadata) -> Result<()> {
     // Deserialize wasm module from a file.
     let mut module =
-        parity_wasm::deserialize_file(&crate_metadata.original_wasm).context(format!(
-            "Loading original wasm file '{}'",
-            crate_metadata.original_wasm.display()
-        ))?;
+        load_module(&crate_metadata.original_wasm).context("Loading of original wasm failed")?;
 
-    // Perform optimization.
-    //
-    // In practice only tree-shaking is performed, i.e transitively removing all symbols that are
-    // NOT used by the specified entrypoints.
-    if pwasm_utils::optimize(&mut module, ["call", "deploy"].to_vec()).is_err() {
-        anyhow::bail!("Optimizer failed");
-    }
+    strip_exports(&mut module);
     ensure_maximum_memory_pages(&mut module, MAX_MEMORY_PAGES)?;
     strip_custom_sections(&mut module);
 
@@ -306,6 +382,7 @@ fn post_process_wasm(crate_metadata: &CrateMetadata) -> Result<()> {
 fn optimize_wasm(
     crate_metadata: &CrateMetadata,
     optimization_passes: OptimizationPasses,
+    keep_debug_symbols: bool,
 ) -> Result<OptimizationResult> {
     let mut dest_optimized = crate_metadata.dest_wasm.clone();
     dest_optimized.set_file_name(format!(
@@ -314,8 +391,9 @@ fn optimize_wasm(
     ));
     let _ = do_optimization(
         crate_metadata.dest_wasm.as_os_str(),
-        &dest_optimized.as_os_str(),
+        dest_optimized.as_os_str(),
         optimization_passes,
+        keep_debug_symbols,
     )?;
 
     if !dest_optimized.exists() {
@@ -348,6 +426,7 @@ fn do_optimization(
     dest_wasm: &OsStr,
     dest_optimized: &OsStr,
     optimization_level: OptimizationPasses,
+    keep_debug_symbols: bool,
 ) -> Result<()> {
     // check `wasm-opt` is installed
     let which = which::which("wasm-opt");
@@ -379,7 +458,8 @@ fn do_optimization(
         "Optimization level passed to wasm-opt: {}",
         optimization_level
     );
-    let output = Command::new(wasm_opt_path)
+    let mut command = Command::new(wasm_opt_path);
+    command
         .arg(dest_wasm)
         .arg(format!("-O{}", optimization_level))
         .arg("-o")
@@ -387,15 +467,18 @@ fn do_optimization(
         // the memory in our module is imported, `wasm-opt` needs to be told that
         // the memory is initialized to zeroes, otherwise it won't run the
         // memory-packing pre-pass.
-        .arg("--zero-filled-memory")
-        .output()
-        .map_err(|err| {
-            anyhow::anyhow!(
-                "Executing {} failed with {:?}",
-                wasm_opt_path.display(),
-                err
-            )
-        })?;
+        .arg("--zero-filled-memory");
+    if keep_debug_symbols {
+        command.arg("-g");
+    }
+    log::info!("Invoking wasm-opt with {:?}", command);
+    let output = command.output().map_err(|err| {
+        anyhow::anyhow!(
+            "Executing {} failed with {:?}",
+            wasm_opt_path.display(),
+            err
+        )
+    })?;
 
     if !output.status.success() {
         let err = str::from_utf8(&output.stderr)
@@ -520,19 +603,41 @@ fn assert_compatible_ink_dependencies(
     Ok(())
 }
 
+/// Checks whether the supplied `ink_version` already contains the debug feature.
+///
+/// This feature was introduced in `3.0.0-rc4` with `ink_env/ink-debug`.
+pub fn assert_debug_mode_supported(ink_version: &Version) -> anyhow::Result<()> {
+    log::info!("Contract version: {:?}", ink_version);
+    let minimum_version = Version::parse("3.0.0-rc4").expect("parsing version failed");
+    if ink_version < &minimum_version {
+        anyhow::bail!(
+            "Building the contract in debug mode requires an ink! version newer than `3.0.0-rc3`!"
+        );
+    }
+    Ok(())
+}
+
 /// Executes build of the smart-contract which produces a wasm binary that is ready for deploying.
 ///
 /// It does so by invoking `cargo build` and then post processing the final binary.
-pub(crate) fn execute(
-    manifest_path: &ManifestPath,
-    verbosity: Verbosity,
-    build_artifact: BuildArtifacts,
-    unstable_flags: UnstableFlags,
-    optimization_passes: OptimizationPasses,
-) -> Result<BuildResult> {
+pub(crate) fn execute(args: ExecuteArgs) -> Result<BuildResult> {
+    let ExecuteArgs {
+        manifest_path,
+        verbosity,
+        build_mode,
+        build_artifact,
+        unstable_flags,
+        optimization_passes,
+        keep_debug_symbols,
+        output_type,
+    } = args;
+
     let crate_metadata = CrateMetadata::collect(&manifest_path)?;
 
     assert_compatible_ink_dependencies(&manifest_path, verbosity)?;
+    if build_mode == BuildMode::Debug {
+        assert_debug_mode_supported(&crate_metadata.ink_version)?;
+    }
 
     let build = || -> Result<OptimizationResult> {
         maybe_println!(
@@ -541,7 +646,13 @@ pub(crate) fn execute(
             format!("[1/{}]", build_artifact.steps()).bold(),
             "Building cargo project".bright_green().bold()
         );
-        exec_cargo_for_wasm_target(&crate_metadata, "build", verbosity, &unstable_flags)?;
+        exec_cargo_for_wasm_target(
+            &crate_metadata,
+            "build",
+            build_mode,
+            verbosity,
+            &unstable_flags,
+        )?;
 
         maybe_println!(
             verbosity,
@@ -557,14 +668,21 @@ pub(crate) fn execute(
             format!("[3/{}]", build_artifact.steps()).bold(),
             "Optimizing wasm file".bright_green().bold()
         );
-        let optimization_result = optimize_wasm(&crate_metadata, optimization_passes)?;
+        let optimization_result =
+            optimize_wasm(&crate_metadata, optimization_passes, keep_debug_symbols)?;
 
         Ok(optimization_result)
     };
 
     let (opt_result, metadata_result) = match build_artifact {
         BuildArtifacts::CheckOnly => {
-            exec_cargo_for_wasm_target(&crate_metadata, "check", verbosity, &unstable_flags)?;
+            exec_cargo_for_wasm_target(
+                &crate_metadata,
+                "check",
+                BuildMode::Release,
+                verbosity,
+                &unstable_flags,
+            )?;
             (None, None)
         }
         BuildArtifacts::CodeOnly => {
@@ -585,27 +703,34 @@ pub(crate) fn execute(
         }
     };
     let dest_wasm = opt_result.as_ref().map(|r| r.dest_wasm.clone());
+
     Ok(BuildResult {
         dest_wasm,
         metadata_result,
         target_directory: crate_metadata.target_directory,
         optimization_result: opt_result,
+        build_mode,
         build_artifact,
         verbosity,
+        output_type,
     })
 }
 
 #[cfg(feature = "test-ci-only")]
 #[cfg(test)]
 mod tests_ci_only {
-    use super::{assert_compatible_ink_dependencies, check_wasm_opt_version_compatibility};
+    use super::{
+        assert_compatible_ink_dependencies, assert_debug_mode_supported,
+        check_wasm_opt_version_compatibility,
+    };
     use crate::{
-        cmd::BuildCommand,
+        cmd::{build::load_module, BuildCommand},
         util::tests::{with_new_contract_project, with_tmp_dir},
         workspace::Manifest,
-        BuildArtifacts, ManifestPath, OptimizationPasses, UnstableFlags, UnstableOptions,
+        BuildArtifacts, BuildMode, ManifestPath, OptimizationPasses, OutputType, UnstableOptions,
         Verbosity, VerbosityFlags,
     };
+    use semver::Version;
     #[cfg(unix)]
     use std::os::unix::fs::PermissionsExt;
     use std::{
@@ -616,10 +741,7 @@ mod tests_ci_only {
 
     /// Modifies the `Cargo.toml` under the supplied `cargo_toml_path` by
     /// setting `optimization-passes` in `[package.metadata.contract]` to `passes`.
-    fn write_optimization_passes_into_manifest(
-        cargo_toml_path: &PathBuf,
-        passes: OptimizationPasses,
-    ) {
+    fn write_optimization_passes_into_manifest(cargo_toml_path: &Path, passes: OptimizationPasses) {
         let manifest_path =
             ManifestPath::new(cargo_toml_path).expect("manifest path creation failed");
         let mut manifest = Manifest::new(manifest_path.clone()).expect("manifest creation failed");
@@ -631,6 +753,13 @@ mod tests_ci_only {
             .expect("writing manifest failed");
     }
 
+    fn has_debug_symbols<P: AsRef<Path>>(p: P) -> bool {
+        load_module(p)
+            .unwrap()
+            .custom_sections()
+            .any(|e| e.name() == "name")
+    }
+
     /// Creates an executable `wasm-opt-mocked` file which outputs
     /// "wasm-opt version `version`".
     ///
@@ -654,14 +783,14 @@ mod tests_ci_only {
     #[test]
     fn build_code_only() {
         with_new_contract_project(|manifest_path| {
-            let res = super::execute(
-                &manifest_path,
-                Verbosity::Default,
-                BuildArtifacts::CodeOnly,
-                UnstableFlags::default(),
-                OptimizationPasses::default(),
-            )
-            .expect("build failed");
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_mode: BuildMode::Release,
+                build_artifact: BuildArtifacts::CodeOnly,
+                ..Default::default()
+            };
+
+            let res = super::execute(args).expect("build failed");
 
             // our ci has set `CARGO_TARGET_DIR` to cache artifacts.
             // this dir does not include `/target/` as a path, hence
@@ -682,6 +811,10 @@ mod tests_ci_only {
             // our optimized contract template should always be below 3k.
             assert!(optimized_size < 3.0);
 
+            // we specified that debug symbols should be removed
+            // original code should have some but the optimized version should have them removed
+            assert!(!has_debug_symbols(&res.dest_wasm.unwrap()));
+
             Ok(())
         })
     }
@@ -691,16 +824,14 @@ mod tests_ci_only {
         with_new_contract_project(|manifest_path| {
             // given
             let project_dir = manifest_path.directory().expect("directory must exist");
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path: manifest_path.clone(),
+                build_artifact: BuildArtifacts::CheckOnly,
+                ..Default::default()
+            };
 
             // when
-            super::execute(
-                &manifest_path,
-                Verbosity::Default,
-                BuildArtifacts::CheckOnly,
-                UnstableFlags::default(),
-                OptimizationPasses::default(),
-            )
-            .expect("build failed");
+            super::execute(args).expect("build failed");
 
             // then
             assert!(
@@ -720,17 +851,20 @@ mod tests_ci_only {
         with_new_contract_project(|manifest_path| {
             // given
             write_optimization_passes_into_manifest(
-                &manifest_path.clone().into(),
+                manifest_path.as_ref(),
                 OptimizationPasses::Three,
             );
             let cmd = BuildCommand {
                 manifest_path: Some(manifest_path.into()),
                 build_artifact: BuildArtifacts::All,
+                build_release: false,
                 verbosity: VerbosityFlags::default(),
                 unstable_options: UnstableOptions::default(),
 
                 // we choose zero optimization passes as the "cli" parameter
                 optimization_passes: Some(OptimizationPasses::Zero),
+                keep_debug_symbols: false,
+                output_json: false,
             };
 
             // when
@@ -740,17 +874,14 @@ mod tests_ci_only {
                 .expect("no optimization result available");
 
             // then
-            // we have to truncate here to account for a possible small delta
-            // in the floating point numbers
-            let optimized_size = optimization.optimized_size.trunc();
-            let original_size = optimization.original_size.trunc();
+            // The size does not exactly match the original size even without optimization
+            // passed because there is still some post processing happening.
+            let size_diff = optimization.original_size - optimization.optimized_size;
             assert!(
-                optimized_size == original_size,
-                "The optimized size {:?} differs from the original size {:?}",
-                optimized_size,
-                original_size
+                0.0 < size_diff && size_diff < 10.0,
+                "The optimized size savings are larger than allowed or negative: {}",
+                size_diff,
             );
-
             Ok(())
         })
     }
@@ -760,17 +891,20 @@ mod tests_ci_only {
         with_new_contract_project(|manifest_path| {
             // given
             write_optimization_passes_into_manifest(
-                &manifest_path.clone().into(),
+                manifest_path.as_ref(),
                 OptimizationPasses::Three,
             );
             let cmd = BuildCommand {
                 manifest_path: Some(manifest_path.into()),
                 build_artifact: BuildArtifacts::All,
+                build_release: false,
                 verbosity: VerbosityFlags::default(),
                 unstable_options: UnstableOptions::default(),
 
                 // we choose no optimization passes as the "cli" parameter
                 optimization_passes: None,
+                keep_debug_symbols: false,
+                output_json: false,
             };
 
             // when
@@ -780,15 +914,13 @@ mod tests_ci_only {
                 .expect("no optimization result available");
 
             // then
-            // we have to truncate here to account for a possible small delta
-            // in the floating point numbers
-            let optimized_size = optimization.optimized_size.trunc();
-            let original_size = optimization.original_size.trunc();
+            // The size does not exactly match the original size even without optimization
+            // passed because there is still some post processing happening.
+            let size_diff = optimization.original_size - optimization.optimized_size;
             assert!(
-                optimized_size < original_size,
-                "The optimized size DOES NOT {:?} differ from the original size {:?}",
-                optimized_size,
-                original_size
+                size_diff > (optimization.original_size / 2.0),
+                "The optimized size savings are too small: {}",
+                size_diff,
             );
 
             Ok(())
@@ -847,8 +979,13 @@ mod tests_ci_only {
 
             // then
             assert!(res.is_err());
-            assert!(format!("{:?}", res)
-                .starts_with("Err(Your wasm-opt version is 98, but we require a version >= 99."));
+            assert!(
+                format!("{:?}", res).starts_with(
+                    "Err(Your wasm-opt version is 98, but we require a version >= 99."
+                ),
+                "Expected a different output, found {:?}",
+                res
+            );
 
             Ok(())
         })
@@ -927,9 +1064,12 @@ mod tests_ci_only {
             let cmd = BuildCommand {
                 manifest_path: Some(manifest_path.into()),
                 build_artifact: BuildArtifacts::All,
+                build_release: false,
                 verbosity: VerbosityFlags::default(),
                 unstable_options: UnstableOptions::default(),
                 optimization_passes: None,
+                keep_debug_symbols: false,
+                output_json: false,
             };
             let res = cmd.exec().expect("build failed");
 
@@ -944,4 +1084,157 @@ mod tests_ci_only {
             Ok(())
         })
     }
+
+    #[test]
+    pub fn debug_mode_must_be_compatible() {
+        let _ =
+            assert_debug_mode_supported(&Version::parse("3.0.0-rc4").expect("parsing must work"))
+                .expect("debug mode must be compatible");
+        let _ =
+            assert_debug_mode_supported(&Version::parse("4.0.0-rc1").expect("parsing must work"))
+                .expect("debug mode must be compatible");
+        let _ = assert_debug_mode_supported(&Version::parse("5.0.0").expect("parsing must work"))
+            .expect("debug mode must be compatible");
+    }
+
+    #[test]
+    pub fn debug_mode_must_be_incompatible() {
+        let res =
+            assert_debug_mode_supported(&Version::parse("3.0.0-rc3").expect("parsing must work"))
+                .expect_err("assertion must fail");
+        assert_eq!(
+            res.to_string(),
+            "Building the contract in debug mode requires an ink! version newer than `3.0.0-rc3`!"
+        );
+    }
+
+    #[test]
+    fn building_template_in_debug_mode_must_work() {
+        with_new_contract_project(|manifest_path| {
+            // given
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_mode: BuildMode::Debug,
+                ..Default::default()
+            };
+
+            // when
+            let res = super::execute(args);
+
+            // then
+            assert!(res.is_ok(), "building template in debug mode failed!");
+            Ok(())
+        })
+    }
+
+    #[test]
+    fn building_template_in_release_mode_must_work() {
+        with_new_contract_project(|manifest_path| {
+            // given
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_mode: BuildMode::Release,
+                ..Default::default()
+            };
+
+            // when
+            let res = super::execute(args);
+
+            // then
+            assert!(res.is_ok(), "building template in release mode failed!");
+            Ok(())
+        })
+    }
+
+    #[test]
+    fn building_contract_with_source_file_in_subfolder_must_work() {
+        with_new_contract_project(|manifest_path| {
+            // given
+            let path = manifest_path.directory().expect("dir must exist");
+            let old_lib_path = path.join(Path::new("lib.rs"));
+            let new_lib_path = path.join(Path::new("srcfoo")).join(Path::new("lib.rs"));
+            let new_dir_path = path.join(Path::new("srcfoo"));
+            std::fs::create_dir_all(new_dir_path).expect("creating dir must work");
+            std::fs::rename(old_lib_path, new_lib_path).expect("moving file must work");
+
+            let mut manifest =
+                Manifest::new(manifest_path.clone()).expect("creating manifest must work");
+            manifest
+                .set_lib_path("srcfoo/lib.rs")
+                .expect("setting lib path must work");
+            manifest.write(&manifest_path).expect("writing must work");
+
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_artifact: BuildArtifacts::CheckOnly,
+                ..Default::default()
+            };
+
+            // when
+            let res = super::execute(args);
+
+            // then
+            assert!(res.is_ok(), "building contract failed!");
+            Ok(())
+        })
+    }
+
+    #[test]
+    fn keep_debug_symbols_in_debug_mode() {
+        with_new_contract_project(|manifest_path| {
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_mode: BuildMode::Debug,
+                build_artifact: BuildArtifacts::CodeOnly,
+                keep_debug_symbols: true,
+                ..Default::default()
+            };
+
+            let res = super::execute(args).expect("build failed");
+
+            // we specified that debug symbols should be kept
+            assert!(has_debug_symbols(&res.dest_wasm.unwrap()));
+
+            Ok(())
+        })
+    }
+
+    #[test]
+    fn keep_debug_symbols_in_release_mode() {
+        with_new_contract_project(|manifest_path| {
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_mode: BuildMode::Release,
+                build_artifact: BuildArtifacts::CodeOnly,
+                keep_debug_symbols: true,
+                ..Default::default()
+            };
+
+            let res = super::execute(args).expect("build failed");
+
+            // we specified that debug symbols should be kept
+            assert!(has_debug_symbols(&res.dest_wasm.unwrap()));
+
+            Ok(())
+        })
+    }
+
+    #[test]
+    fn build_with_json_output_works() {
+        with_new_contract_project(|manifest_path| {
+            // given
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                output_type: OutputType::Json,
+                ..Default::default()
+            };
+
+            // when
+            let res = super::execute(args).expect("build failed");
+
+            // then
+            assert!(res.serialize_json().is_ok());
+            Ok(())
+        })
+    }
 }
