@@ -22,7 +22,7 @@ mod workspace;
 
 use self::workspace::ManifestPath;
 
-use crate::cmd::{metadata::MetadataResult, BuildCommand, CheckCommand};
+use crate::cmd::{metadata::MetadataResult, BuildCommand, CheckCommand, TestCommand};
 
 #[cfg(feature = "extrinsics")]
 use sp_core::{crypto::Pair, sr25519, H256};
@@ -128,7 +128,7 @@ impl Display for OptimizationPasses {
 
 impl Default for OptimizationPasses {
     fn default() -> OptimizationPasses {
-        OptimizationPasses::Three
+        OptimizationPasses::Z
     }
 }
 
@@ -170,7 +170,7 @@ pub struct VerbosityFlags {
 }
 
 /// Denotes if output should be printed to stdout.
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, serde::Serialize)]
 pub enum Verbosity {
     /// Use default output
     Default,
@@ -180,6 +180,12 @@ pub enum Verbosity {
     Verbose,
 }
 
+impl Default for Verbosity {
+    fn default() -> Self {
+        Verbosity::Default
+    }
+}
+
 impl Verbosity {
     /// Returns `true` if output should be printed (i.e. verbose output is set).
     pub(crate) fn is_verbose(&self) -> bool {
@@ -235,7 +241,7 @@ impl TryFrom<&UnstableOptions> for UnstableFlags {
 }
 
 /// Describes which artifacts to generate
-#[derive(Copy, Clone, Eq, PartialEq, Debug, StructOpt)]
+#[derive(Copy, Clone, Eq, PartialEq, Debug, StructOpt, serde::Serialize)]
 #[structopt(name = "build-artifacts")]
 pub enum BuildArtifacts {
     /// Generate the Wasm, the metadata and a bundled `<name>.contract` file
@@ -270,7 +276,52 @@ impl std::str::FromStr for BuildArtifacts {
     }
 }
 
+impl Default for BuildArtifacts {
+    fn default() -> Self {
+        BuildArtifacts::All
+    }
+}
+
+/// The mode to build the contract in.
+#[derive(Eq, PartialEq, Copy, Clone, Debug, serde::Serialize)]
+pub enum BuildMode {
+    /// Functionality to output debug messages is build into the contract.
+    Debug,
+    /// The contract is build without any debugging functionality.
+    Release,
+}
+
+impl Default for BuildMode {
+    fn default() -> BuildMode {
+        BuildMode::Debug
+    }
+}
+
+impl Display for BuildMode {
+    fn fmt(&self, f: &mut Formatter<'_>) -> DisplayResult {
+        match self {
+            Self::Debug => write!(f, "debug"),
+            Self::Release => write!(f, "release"),
+        }
+    }
+}
+
+/// The type of output to display at the end of a build.
+pub enum OutputType {
+    /// Output build results in a human readable format.
+    HumanReadable,
+    /// Output the build results JSON formatted.
+    Json,
+}
+
+impl Default for OutputType {
+    fn default() -> Self {
+        OutputType::HumanReadable
+    }
+}
+
 /// Result of the metadata generation process.
+#[derive(serde::Serialize)]
 pub struct BuildResult {
     /// Path to the resulting Wasm file.
     pub dest_wasm: Option<PathBuf>,
@@ -280,13 +331,19 @@ pub struct BuildResult {
     pub target_directory: PathBuf,
     /// If existent the result of the optimization.
     pub optimization_result: Option<OptimizationResult>,
+    /// The mode to build the contract in.
+    pub build_mode: BuildMode,
     /// Which build artifacts were generated.
     pub build_artifact: BuildArtifacts,
     /// The verbosity flags.
     pub verbosity: Verbosity,
+    /// The type of formatting to use for the build output.
+    #[serde(skip_serializing)]
+    pub output_type: OutputType,
 }
 
 /// Result of the optimization process.
+#[derive(serde::Serialize)]
 pub struct OptimizationResult {
     /// The path of the optimized wasm file.
     pub dest_wasm: PathBuf,
@@ -309,10 +366,16 @@ impl BuildResult {
             "optimized file size must be greater 0"
         );
 
+        let build_mode = format!(
+            "The contract was built in {} mode.\n\n",
+            format!("{}", self.build_mode).to_uppercase().bold(),
+        );
+
         if self.build_artifact == BuildArtifacts::CodeOnly {
             let out = format!(
-                "{}Your contract's code is ready. You can find it here:\n{}",
+                "{}{}Your contract's code is ready. You can find it here:\n{}",
                 size_diff,
+                build_mode,
                 self.dest_wasm
                     .as_ref()
                     .expect("wasm path must exist")
@@ -324,8 +387,9 @@ impl BuildResult {
         };
 
         let mut out = format!(
-            "{}Your contract artifacts are ready. You can find them in:\n{}\n\n",
+            "{}{}Your contract artifacts are ready. You can find them in:\n{}\n\n",
             size_diff,
+            build_mode,
             self.target_directory.display().to_string().bold(),
         );
         if let Some(metadata_result) = self.metadata_result.as_ref() {
@@ -338,7 +402,7 @@ impl BuildResult {
         if let Some(dest_wasm) = self.dest_wasm.as_ref() {
             let wasm = format!(
                 "  - {} (the contract's code)\n",
-                util::base_name(&dest_wasm).bold()
+                util::base_name(dest_wasm).bold()
             );
             out.push_str(&wasm);
         }
@@ -362,6 +426,11 @@ impl BuildResult {
             .expect("optimization result must exist");
         (optimization.original_size, optimization.optimized_size)
     }
+
+    /// Display the build results in a pretty formatted JSON string.
+    pub fn serialize_json(&self) -> Result<String> {
+        Ok(serde_json::to_string_pretty(self)?)
+    }
 }
 
 #[derive(Debug, StructOpt)]
@@ -383,7 +452,7 @@ enum Command {
     Check(CheckCommand),
     /// Test the smart contract off-chain
     #[structopt(name = "test")]
-    Test {},
+    Test(TestCommand),
     /// Upload the smart contract code to the chain
     #[cfg(feature = "extrinsics")]
     #[structopt(name = "deploy")]
@@ -452,7 +521,10 @@ fn exec(cmd: Command) -> Result<Option<String>> {
         Command::New { name, target_dir } => cmd::new::execute(name, target_dir.as_ref()),
         Command::Build(build) => {
             let result = build.exec()?;
-            if result.verbosity.is_verbose() {
+
+            if matches!(result.output_type, OutputType::Json) {
+                Ok(Some(result.serialize_json()?))
+            } else if result.verbosity.is_verbose() {
                 Ok(Some(result.display()))
             } else {
                 Ok(None)
@@ -472,7 +544,14 @@ fn exec(cmd: Command) -> Result<Option<String>> {
                 Ok(None)
             }
         }
-        Command::Test {} => Err(anyhow::anyhow!("Command unimplemented")),
+        Command::Test(test) => {
+            let res = test.exec()?;
+            if res.verbosity.is_verbose() {
+                Ok(Some(res.display()?))
+            } else {
+                Ok(None)
+            }
+        }
         #[cfg(feature = "extrinsics")]
         Command::Deploy {
             extrinsic_opts,
@@ -500,3 +579,54 @@ fn exec(cmd: Command) -> Result<Option<String>> {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn build_result_seralization_sanity_check() {
+        // given
+        let raw_result = r#"{
+  "dest_wasm": "/path/to/contract.wasm",
+  "metadata_result": {
+    "dest_metadata": "/path/to/metadata.json",
+    "dest_bundle": "/path/to/contract.contract"
+  },
+  "target_directory": "/path/to/target",
+  "optimization_result": {
+    "dest_wasm": "/path/to/contract.wasm",
+    "original_size": 64.0,
+    "optimized_size": 32.0
+  },
+  "build_mode": "Debug",
+  "build_artifact": "All",
+  "verbosity": "Quiet"
+}"#;
+
+        let build_result = crate::BuildResult {
+            dest_wasm: Some(PathBuf::from("/path/to/contract.wasm")),
+            metadata_result: Some(crate::cmd::metadata::MetadataResult {
+                dest_metadata: PathBuf::from("/path/to/metadata.json"),
+                dest_bundle: PathBuf::from("/path/to/contract.contract"),
+            }),
+            target_directory: PathBuf::from("/path/to/target"),
+            optimization_result: Some(crate::OptimizationResult {
+                dest_wasm: PathBuf::from("/path/to/contract.wasm"),
+                original_size: 64.0,
+                optimized_size: 32.0,
+            }),
+            build_mode: Default::default(),
+            build_artifact: Default::default(),
+            verbosity: Verbosity::Quiet,
+            output_type: OutputType::Json,
+        };
+
+        // when
+        let serialized_result = build_result.serialize_json();
+
+        // then
+        assert!(serialized_result.is_ok());
+        assert_eq!(serialized_result.unwrap(), raw_result);
+    }
+}
